/*
        SlideObject.h

	SlideObject.h : This file is part of SlideShow

	Copyright (C) 2004 Fabien VALLON
	Additional copyrights here

	Authors: Fabien VALLON <fabien.vallon@fr.alcove.com>
	Date:	13 October 2004

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA
*/


#ifndef _IMAGEAPPS_SLIDESHOW_SLIDEOBJECT_H_
#define _IMAGEAPPS_SLIDESHOW_SLIDEOBJECT_H__

#include <Foundation/NSObject.h>
@class NSString;
@class NSImage;

@interface SlideObject: NSObject 
{
@private 
  NSString *_path;
  BOOL _mark;
  NSImage  *_image;
}

-(void) setPath: (NSString *) path;
-(NSString *) path;

-(NSString *) description;
-(NSString *) shortName;

-(BOOL) isMark;
-(void) setMark:(BOOL) flag;

@end

#endif /* _IMAGEAPPS_SLIDESHOW_SLIDEOBJECT_H__ */
