/*
        PreferencesController.h

	PreferencesController : This file is part of SlideShow

	Copyright (C) 2004 Fabien VALLON
	Additional copyrights here

	Authors: Fabien VALLON <fabien@sonappart.net>
	Date:	17 August 2004

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA
*/

#ifndef _IMAGEAPPS_SLIDESHOW_PREFERENCECONTROLLER_H_
#define _IMAGEAPPS_SLIDESHOW_PREFERENCECONTROLLER_H_

#include <Foundation/NSObject.h>
#include <AppKit/NSNibDeclarations.h>

#include "Shortcuts.h"

@class NSButton;
@class NSBrowser;
@class NSColorWell;
@class NSPopUpButton;
@class NSTabView;
@class NSPanel;

@interface PreferencesController : NSObject
{ 
  IBOutlet NSPanel *window;
  IBOutlet NSPopUpButton *popUpButton;
  IBOutlet NSTabView *tabView;

  //IBOutlet ( general ) 
  IBOutlet NSColorWell *bgColorWell;
  IBOutlet NSButton *startFullScreenSwitch;
  
  //IBOutlet ( shortcut ) 
  IBOutlet NSBrowser *browser;
  IBOutlet NSButton *catpureButton;
  IBOutlet NSButton *deleteButton;

  //Input 
  IBOutlet NSPanel *inputPanel;
  IBOutlet id      inputView;
  
@private 
  Shortcuts *_shortcuts;
}

+(id) sharedPreferences;

//IBActions

//main popUp
-(IBAction) popUpAction:(id) sender; 
//General
-(IBAction) colorAction: (id) sender;
-(IBAction) startFullScreenSwitchAction: (id) sender;
//Shortcuts
-(IBAction) keystrokeAction:(id) sender;
-(IBAction) deleteAction:(id) sender;
-(IBAction) browserAction:(id) sender;

//browser Delegate methods
- (NSString *)browser:(NSBrowser *)sender titleOfColumn:(int)column;
- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column;
- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column;

//Accessor
-(NSPanel *) window;

@end

#endif /* _IMAGEAPPS_SLIDESHOW_PREFERENCECONTROLLER_H_ */

