/*
        Shortcuts.h

	FullScreenWindow : This file is part of SlideShow

	Copyright (C) 2004 Fabien VALLON
	Additional copyrights here

	Authors: Fabien VALLON <fabien.vallon@fr.alcove.com>
	Date:	24 March 2004

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA
*/

#ifndef _IMAGEAPPS_SLIDESHOW_SHORTCUTS_H
#define _IMAGEAPPS_SLIDESHOW_SHORTCUTS_H

#include <Foundation/NSObject.h>

@class NSMutableDictionary;
@class NSMutableArray;
@class NSString;

@interface Shortcuts: NSObject
{
@private
  NSMutableDictionary *_keyBindingDict;
  NSMutableArray *_actions;
  NSString *_userPath;
}

+(id) sharedShortcuts;

-(NSArray *) actions;
-(void) setShortcut:(NSString*) shortcut
	   forAction: (NSString*) action;
-(void) removeShortcut: (NSString*) shortcuts;
-(SEL) actionForKey:(NSString*) key;
-(NSArray *) shortcutsForAction: (NSString *) action;

-(void) synchronize;

@end

#endif
