#ifndef _IMAGEAPPS_SLIDESHOW_CACHEENGINE_H_
#define _IMAGEAPPS_SLIDESHOW_CACHEENGINE_H_


#include <Foundation/NSObject.h>
#include <AppKit/AppKit.h>

#include "SlideObject.h"

@protocol CacheEngineMethods

-(void) removePath:(NSString *) path;
-(void) addImagePaths: (NSArray *) paths;
-(NSData *) imageForPath:(NSString *) path;
-(oneway void) userUsePath: (NSString *) path;

@end

@interface CacheEngine: NSObject <CacheEngineMethods>
{
  NSString *_string;
  NSMutableDictionary *_backend;
}

+ (void)connectWithPorts:(NSArray *)portArray;

-(void) removePath:(NSString *) path;
-(void) addImagePaths: (NSArray *) paths;
-(void) userUsePath: (NSString *) path;
-(NSData *) imageForPath:(NSString *) path;

@end

#endif /* _IMAGEAPPS_SLIDESHOW_CACHEENGINE_H_ */
