/*
        main.m

	main : This file is part of SlideShow

	Copyright (C)2004 Fabien VALLON
	Additional copyrights here

	Authors: Fabien VALLON <fabien@tuxfamily.org>
	Date:	23 July 2004

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA
*/


#include <AppKit/NSApplication.h>

int main(int argc, const char *argv[]) 
{
  return NSApplicationMain (argc, argv);
}



